@@ -61,13 +61,13 @@ class Netlas:
         except orjson.JSONDecodeError:
             ret["error"] = "Failed to parse response data to JSON"
             if self.debug:
-                ret["error_description"] = r.reason
-                ret["error_data"] = r.text
+                ret["error"] += "\nDescription: " + r.reason
+                ret["error"] += "\nData: " + r.text
         except requests.HTTPError:
             ret["error"] = f"{r.status_code}: {r.reason}"
             if self.debug:
-                ret["error_description"] = r.reason
-                ret["error_data"] = r.text
+                ret["error"] += "\nDescription: " + r.reason
+                ret["error"] += "\nData: " + r.text
 
         if ret.get('error', None):
             raise APIError(ret['error'])
@@ -96,26 +96,26 @@ class Netlas:
             ret["error"] = "API key is empty"
             raise APIError(ret['error'])
         try:
-            with requests.get(f"{self.apibase}{endpoint}",
-                              params=params,
-                              headers=self.headers,
-                              verify=self.verify_ssl,
-                              stream=True) as r:
+            with requests.post(
+                    f"{self.apibase}{endpoint}",
+                    json=params,
+                    headers=self.headers,
+                    verify=self.verify_ssl,
+                    stream=True,
+            ) as r:
                 check_status_code(request=r, debug=self.debug, ret=ret)
-                for chunk in r.iter_content(chunk_size=2048):
+                for chunk in r.iter_lines():
                     #skip keep-alive chunks
                     if chunk:
                         yield chunk
-        except requests.HTTPError:
-            ret["error"] = f"{r.status_code}: {r.reason}"
-            if self.debug:
-                ret["error_description"] = r.reason
-                ret["error_data"] = r.text
+        except:
+            ret["error"] = f"Unexpected Stream error"
             raise APIError(ret['error'])
 
     def query(self,
               query: str,
               datatype: str = "response",
+              page: int = 0,
               indices: str = "") -> dict:
         """Send search query to Netlas API
 
@@ -123,6 +123,8 @@ class Netlas:
         :type query: str
         :param datatype: Data type (choises: response, cert, domain), defaults to "response"
         :type datatype: str, optional
+        :param page: Page number of data, defaults to 0
+        :type page: int, optional
         :param indices: Comma-separated IDs of selected data indices (can be retrieved by `indices` method), defaults to ""
         :type indices: str, optional
         :return: search query result
@@ -137,7 +139,8 @@ class Netlas:
             endpoint=endpoint,
             params={
                 "q": query,
-                "indices": indices
+                "indices": indices,
+                "start": page * 20,
             },
         )
         return ret
@@ -222,21 +225,29 @@ class Netlas:
         )
         return ret
 
-    def download(self,
-                 query: str,
-                 datatype: str = "response",
-                 size: int = 10,
-                 indices: str = "") -> bytes:
+    def download(
+                self,
+                query: str,
+                fields: list = list(),
+                source_type: str = "include",
+                datatype: str = "response",
+                size: int = 10,
+                indices: str = "",
+    ) -> bytes:
         """Download data from Netlas
 
         :param query: Search query string
         :type query: str
-        :param datatype: Data type (choises: response, cert, domain), defaults to "response"
+        :param fields: Comma-separated list of fields to include/exclude, defaults to []
+        :type fields: list
+        :param source_type: Include or exclude fields (choices: include, exclude), defaults to "include"
+        :type source_type: str
+        :param datatype: Data type (choices: response, cert, domain), defaults to "response"
         :type datatype: str, optional
         :param size: Download documents count, defaults to 10
         :type size: int, optional
         :param indices: Comma-separated IDs of selected data indices (can be retrieved by `indices` method), defaults to ""
-        :type indices: str, optional
+        :type indices: list, optional
         :return: Iterator of raw data
         :rtype: Iterator[bytes]
         """
@@ -245,12 +256,18 @@ class Netlas:
             endpoint = "/api/certs/download/"
         elif datatype == "domain":
             endpoint = "/api/domains/download/"
+
+        fields = list() if not fields else fields.split(",")
+
         for ret in self._stream_request(
                 endpoint=endpoint,
                 params={
                     "q": query,
                     "size": size,
-                    "indices": indices
+                    "indices": indices,
+                    "raw": True,
+                    "fields": fields,
+                    "source_type": source_type,
                 },
         ):
             yield ret
@@ -263,4 +280,4 @@ class Netlas:
         """
         endpoint = "/api/indices/"
         ret = self._request(endpoint=endpoint)
-        return ret
\ No newline at end of file
+        return ret
